import click
from click_help_colors import HelpColorsGroup, HelpColorsCommand
from cada.core import Runner

PROG_NAME = 'cada'

@click.command(cls=HelpColorsCommand, help_headers_color='yellow', help_options_color='blue', name=PROG_NAME, context_settings=dict(show_default=True, help_option_names=["-h", "--help"]))
@click.argument('COMMAND_EXPR')
@click.argument('EVAL_EXPR', nargs=-1)
@click.option('-H', '--include-hidden', is_flag=True, help='Include files that starts with period.')
@click.option('-f', '--filter', 'filter_', multiple=True, help='Determines which files to skip.')
@click.option('-s', '--sort-alg', 'sort_alg_name', type=click.Choice(['none', 'simple', 'natural', 'natural-ignore-case']), default='natural-ignore-case', help='Determines execution order.')
@click.option('-k', '--sort-key', help='Key expression used for sorting.')
@click.option('-r', '--reverse', is_flag=True, help='Reverse execution order.')
@click.option('-i', '--import', 'import_', multiple=True, help='Import extra symbol used in eval-expressions.')
@click.option('-d', '--dry-run', is_flag=True, help='Do not execute commands. Only show what would be executed.')
@click.option('-x', '--stop-at-error', is_flag=True, help='Terminates at the first command that returns code other than 0.')
@click.option('-j', '--jobs', type=int, help='Number of concurent jobs that will execute commands. 0 means `auto`.')
@click.option('-q', '--quiet', is_flag=True, help='Do not print anything except stdout and stderr of the executed commands.')
@click.option('--color', type=click.Choice(['always', 'auto', 'never']), default='auto', help='When to use terminal colors.')
@click.version_option(None, '-V', '--version', package_name='para-cada', prog_name=PROG_NAME)
def main(command_expr, eval_expr, **kwargs):
    """Executes your command for each file selected using glob expression(s)."""
    Runner(command_expr, eval_expr, **kwargs).run()

if __name__ == '__main__':
    main()



import sys
import multiprocessing as mp
from colorama import Fore, Style

class PrinterImpl:

    def __init__(self):
        self.stream = sys.stderr
        self.is_tty = self.stream.isatty()
        if not self.is_tty:
            self.clear_line = lambda: None

    def set_monohrome(self):
        self.show_colored = self.show_monohrome

    def show(self, *args, **kwargs):
        print(*args, file=self.stream, **kwargs)

    def show_colored(self, color, *args, **kwargs):
        self.show(*(color + str(a) + Style.RESET_ALL for a in args), **kwargs)
        
    def show_monohrome(self, color, *args, **kwargs):
        self.show(*args, **kwargs)
        
    def show_blue(self, *args, **kwargs):
        self.show_colored(Fore.BLUE, *args, **kwargs)
        
    def show_green(self, *args, **kwargs):
        self.show_colored(Fore.GREEN, *args, **kwargs)
        
    def show_red(self, *args, **kwargs):
        self.show_colored(Fore.RED, *args, **kwargs)
        
    def show_yellow(self, *args, **kwargs):
        self.show_colored(Fore.YELLOW, *args, **kwargs)

    def clear_line(self):
        """move caret to the begining and clear to the end of line"""
        self.show("\r\033[K", end='')


class ReservedPrinter:
    def __init__(self):
        self._lock = mp.Lock()
        self._impl = PrinterImpl()
    
    @property
    def is_tty(self):
        return self._impl.is_tty

    def set_monohrome(self):
        self._impl.set_monohrome()

    def __enter__(self):
        self._lock.acquire()
        return self._impl
    
    def __exit__(self, *args):
        self._impl.stream.flush()
        self._lock.release()



import os
import sys
from pathlib import Path
from importlib import import_module

STARTUP_FILENAME = 'startup.py'

def load_startup_module():
    from cada.__main__ import PROG_NAME
    config_dir_default = Path().home() / ('.' + PROG_NAME)
    config_dir = Path(os.getenv('CADA_CONFIG_DIR', config_dir_default))
    startup_path = config_dir / STARTUP_FILENAME
    if startup_path.exists():
        sys.path.insert(0, str(config_dir))
        import_module(startup_path.stem)

class PluginsRegistry:

    def __init__(self):
        self._plugins = {}

    def register(self, name):
        def register_(f):
            self._plugins[name] = f
        return register_

    def get_instance(self, product):
        inst = {}
        if product:
            inst.update( {k: LazyWrapper(v, product[0]) for k, v in self._plugins.items()} )
            inst.update( {k + str(0): v for k, v in inst.items()} )
            inst.update( {k + str(i): LazyWrapper(v, p) for k, v in self._plugins.items() for i, p in enumerate(product) if i >= 1} )
        return inst


class LazyWrapper:
    def __init__(self, func, arg):
        self._func = func
        self._arg = arg
        
    def _get_instance(self):
        return self._func(self._arg)
        
    def __str__(self):
        return str(self._get_instance())
    
    def __getattr__(self, name):
        return getattr(self._get_instance(), name)


class LazyLocals:
    
    def __init__(self, mapping):
        self._mapping = mapping
    
    def __getitem__(self, name):
        return self._mapping[name]._get_instance()

                
plugins = PluginsRegistry()
symbols = {}
